#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  Helder Fraga aka Whise (c) helderfraga@gmail.com
#  Julien Lavergne (c) 2009 julien.lavergne@gmail.com
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

#TODO
#No todo at the moment

import screenlets
from screenlets import Tooltip, utils
from screenlets.options import StringOption, FontOption, BoolOption, IntOption, ColorOption, DirectoryOption, ListOption
from screenlets.utils import xdg_open,get_filename_on_drop
#from screenlets import DefaultMenuItem 
import cairo
import gtk
import pango
import urllib

import threading
import time

try:
	import gnomedesktop
	GNOMEDESK = True
except:
	GNOMEDESK = False

try:
	import gnome.ui
	GNOMEUI = True
except:
	GNOMEUI = False
	
import os
import gobject
import gc

try:
	import gio
	
except:
	screenlets.show_message(None,'You don\'t have GIO python bindings installed. \nYou need to install python-gobject >= 2.15 .')
	sys.exit()


from xdg.BaseDirectory import *

import subprocess

#===================================================================
#  Tweak Parameters
#===================================================================

# Central folder directory
CENTRAL_FOLDER = os.path.join(os.path.expanduser("~"), ".centralFolder")

def getFolders(dir):
	os.chdir(dir)
	return [d for d in os.listdir(str(dir)) if os.path.isdir(d)]

class MultiFolderViewScreenlet(screenlets.Screenlet):
	
	# default meta-info for Screenlets
	__name__ = 'MultiFolderView'
	__version__ = '0.9.9'
	__author__ = 'Helder Fraga aka Whise'
	__desc__ = 'Display the content of a folder'

	icon_size = 48
	rows = 5
	columns = 1
	pericons = 5
	frame_color = 0.1,0.1,0.1,0.5
	frame_color_selected = 0,0,0,0.5
	color_text_selected =(0, 0, 0, 0.6)
	color_text = 1,1,1,1
	color_title = 1,1,1,1
	banner_size = 0
	folder_path = os.path.join(os.path.expanduser("~"))
	#folder_path_current = os.path.join(os.path.expanduser("~"))
	font = 'Sans 8'
	title_font = 'Sans 12'
	border_size = 1
	border_color=(1,1,1,0.3)
	shadow_size = 1
	shadow_color=(0,0,0,0.5)
	clicked = False
	rounda = 2
	roundb = 2
	sb_row = 5
	sb_column = 3
	icon_num = 0
	show_start = 0
	old_value = 0
	show_end = 16
	files_list_show = []
	fullscroll = 1
	full_path = False
	# internals
	__timeout = None
	exclude = ['Thumbs.db','desktop.ini','__MACOSX','._.DS_Store']
	expand_choices = 'horizontal','vertical'
	expand2 = 'Use a scrollbar'
	expand2_choices = 'Expand/Adjust to icons','Use a scrollbar'
	expand = 'horizontal'
	showtip = False
	showthumb = True
	show_title = True
	files_list_show_reversed = []
	name = ''
	timer1 = None
	image_filename 	= ""
	show_back = False
	cp = ''
	showbyex = ''
	__drag_icon = None
	__drag_icon_mask = None
	__drag_sel = None
	__update_interval = 1 # every second

	# constructor
	def __init__(self, **keyword_args):
		screenlets.Screenlet.__init__(self, width=200, height=200,ask_on_option_override=False, drag_drop=True,resize_on_scroll= False, **keyword_args)
		
		# Stores the number of the MultiFolderView
		self.numId = int(self.id[-1])

		#self.x = self.width
		#self.y = self.height
		# set theme
		self.apps_list = []
		self.__buffer_back = None
		self.__buffer_fore = None
		self.current_file = None
		self.theme_name = "default"
		#Init the theme to search for icons and default style
		self.icontheme = gtk.icon_theme_get_default()
		self.icontheme.connect("changed", self.icons_changed)
		#Default Options
		self.icon_size = 48
		self.columns = 2
		self.files_list_show = []
		self.timer = None

		
		self.folder_path_current = self.folder_path
		self.font = "Sans 8"
		self.title_font = "Sans 12"
		self.border_size_selected = 2

		self.cursor_position = [-1,-1]

		self.auto_update = False

		# add default menu items
		#self.add_menuitem("up_folder", "Up")


		#add custom option
		
		self.add_options_group('Folder', 'Settings for the folder ...')



		self.add_option(StringOption('Folder', 'expand2', self.expand2,'Expand Mode', '',choices = self.expand2_choices))

		self.add_option(IntOption('Folder','banner_size',
						self.banner_size, 'Size of the Banner','', min=0, max=64536
						,hidden=True))

		self.add_option(DirectoryOption('Folder', 'folder_path',
					self.folder_path, 'Folder path', 'Default path to the folder',
					))

		self.add_option(ListOption('Folder', 'exclude', self.exclude,'Exclude list', '',))

		self.add_option(BoolOption('Folder','showtip', self.showtip, 'Display Tooltip', '',))

		self.add_option(BoolOption('Folder','showthumb', self.showthumb, 'Show Thumbnails if available', '',))

		self.add_option(StringOption('Folder','showbyex', self.showbyex, 'Show only by extension', '',))

		self.add_option(BoolOption('Folder','show_title', self.show_title, 'Show Path in Title', '',))

		self.add_option(BoolOption('Folder','full_path', self.full_path, 'Display full path name in banner', '',))


		self.add_options_group('Look', 'Settings colors and fonts')

		self.add_option(IntOption('Look','icon_size',
						self.icon_size, 'Icons Size','', min=0, max=64536
						))

		self.add_option(FontOption('Look','title_font', 
						self.title_font, 'Title Font', '',))

		self.add_option(ColorOption('Look','color_title', 
						self.color_title, 'Title color', '',))

		self.add_option(FontOption('Look','font', 
						self.font, 'Text Font', '',))

		self.add_option(ColorOption('Look','color_text', 
						self.color_text, 'Text color', '',))

		self.add_option(ColorOption('Look','frame_color', 
						self.frame_color, 'Background Color', 'Frame color',))

		self.add_option(IntOption('Look','rounda',
						self.rounda, 'Rounded Corner Angle','',min=0, max=20
						))
		self.add_option(ColorOption('Look','border_color', 
						self.border_color, 'Border color', 'Color of the border'))

		self.add_option(IntOption('Look','border_size',
						self.border_size, 'Border Size','',min=0, max=4
						))
		self.add_option(ColorOption('Look','shadow_color', 
						self.shadow_color, 'Shadow Color', 'Color of the shadow'))
		self.add_option(IntOption('Look','shadow_size',
						self.shadow_size, 'Shadow Size','',min=0, max=4
						))

		self.add_option(ColorOption('Look','frame_color_selected', 
						self.frame_color_selected, 'Selection color', 'Frame color for selected background'))

		self.add_option(IntOption('Look','roundb',
						self.roundb, 'Selected Rounded Angle','',min=0, max=20
						))
		self.add_option(IntOption('Look','border_size_selected',
						self.border_size_selected, 'Selection Border Size','',min=0, max=4
						))
		self.add_option(ColorOption('Look','color_text_selected', 
						self.color_text_selected, 'Selection Text Color', 'Color of the text when selected',hidden=True))


		self.add_options_group('Background', 'Settings colors and fonts')

		self.add_option(BoolOption('Background','show_back', self.show_back, 'Show background image', '',))
		self.add_option(StringOption('Background', 'image_filename', self.image_filename, 'Background image', 'Filename of image to be shown in background')) 
		self.add_options_group('With Scrollbar', 'Settings with scrollbars activated')

		self.add_options_group('Without Scrollbar', 'Settings without scrollbars activated')

		self.add_option(IntOption('With Scrollbar','sb_row',
						self.sb_row, 'Icons per Column','Maximum number of icons in a row', min=1, max=64536
						))

		self.add_option(IntOption('With Scrollbar','sb_column',
						self.sb_column, 'Icons per Row','Maximum number of icons in a column', min=1, max=64536
						))

		self.add_option(StringOption('Without Scrollbar', 'expand', self.expand,'Expand Direction', '',choices = self.expand_choices))

		self.add_option(IntOption('Without Scrollbar','pericons',
						self.pericons, 'Icons per Direction','Number of icons in a row or columns, depends on horizontal/vertical', min=1, max=64536
						))


		self.scrollbarbox = gtk.Fixed()
		self.adjust = gtk.Adjustment(0.0,0.0, 1.0, 1.0, 1.0, 1.0)
		self.scrollbar =  gtk.VScrollbar(self.adjust)
		self.window.add(self.scrollbarbox)
		self.window.connect("motion-notify-event",self.motion_notify_event)
		self.adjust.connect("value-changed", self.scroll_value_changed)
		self.scrollbar.connect("expose-event", self.sb_expose)
		self.window.connect("window_state_event",self.state_event)
		self.window.connect("drag-data-get", self.drag_data_get)
		targets = [('text/uri-list', 0, 0)]
		self.window.drag_source_set(gtk.gdk.BUTTON1_MASK,targets,gtk.gdk.ACTION_COPY)
		# set as text-source
		self.window.drag_source_add_text_targets() 
		#self.window.set_no_show_all(True)
		self.clipboard = gtk.Clipboard(selection='CLIPBOARD') 
		#self.scrollbar.set_no_show_all(True)
		gc.collect()
		self.screen_width = 0
		self.screen_height = 0		

	def state_event(self,widget,event):
		#if gtk.gdk.WINDOW_STATE_ABOVE in widget.window.get_state():
		pass#print widget.window.get_state()

	def get_selected_element(self):
		if self.expand2 == 'Use a scrollbar':
			self.list =  self.files_list_show[self.show_start:self.show_end]
		else:
			self.list =  self.files_list_show
		for elem in self.list:
			if elem[2] == self.cursor_position:
				return elem
	
#-------------------------------------------
# CLIPBOARD METHODS
#-------------------------------------------

	def clipboardGet(self, clipboard, selection_data, info,data):
		#selection_data.set( "text/uri-list", 8, tempImgUri )
		
		selection_data.set('x-special/gnome-copied-files', 8,'copy\n' + self.cp)
		#selection_data.set('text/uri-list', 8,self.cp)



	def _clipboardClearFuncCb( self, clipboard, tempImgPath):
		pass

#-------------------------------------------
# DRAG N DROP METHODS
#-------------------------------------------

	def create_drag_icon (self):
		"""Create drag_icon and drag_mask for drag-operation."""
		elem = self.get_selected_element()
		if elem:
			self.__drag_icon = gtk.gdk.pixbuf_new_from_file_at_size(self.generate_icon_names(elem),int(self.icon_size*self.scale), int(self.icon_size*self.scale))
			self.__drag_sel = elem

	
	def drag_end(self, widget, drag_context):
		# call user-defined handler
		self.__drag_icon = None
		self.on_mouse_leave(1)
	
	def drag_begin(self, widget, drag_context):
		if self.cursor_position != [-1,-1]:
			self.create_drag_icon()
			if self.__drag_icon:
				self.window.drag_source_set_icon_pixbuf(self.__drag_icon)


	def drag_data_get (self, widget, drag_context, selection_data, info, timestamp):

		uri_list = 'file://'+ (self.__drag_sel[0].get_path())
		selection_data.set(selection_data.target, 8,uri_list)

	def on_drop (self, x, y, sel_data, timestamp):
		filename = ''
		if sel_data.get_text() == None:
			if sel_data.get_uris() != None:
				filename = sel_data.get_uris()[0]
				filename = filename.replace('file://', '')
			os.system('cp ' + filename + ' ' + chr(34) + self.folder_path_current + chr(34) + ' &')
		else:
			filename = get_filename_on_drop(sel_data)
		        for f in filename:
		                if f != '':
		                        os.system('cp ' + f + ' ' + chr(34) + self.folder_path_current + chr(34) + ' &')

#-------------------------------------------
# TOOLTIPS METHODS
#-------------------------------------------

	def hide_tip(self):
		if self.showtip:
			self.hide_tooltip()
			if self.timer:
				gobject.source_remove(self.timer)
				self.timer = None


	def show_tip(self):
		if self.showtip:
			self.timer = gobject.timeout_add(2000, self.show_tips)



#-------------------------------------------
# SCROLLBARS METHODS
#-------------------------------------------

	def sb_expose(self,widget,event):
		if self.fullscroll <= 1: 
			widget.hide()


	def scroll_value_changed(self,widget):

		self.value = self.adjust.get_value()
		if len(str(self.value)) < 8:
			self.adjust.set_value(int(self.value))
			self.show_start = int(self.value * self.sb_row* self.sb_column)
			self.show_end = int(self.value * self.sb_row* self.sb_column + (self.sb_row* self.sb_column))
			if self.show_end > self.icon_num: self.show_end = self.icon_num
			
			
			self.redraw_foreground()
			self.update()
		
		else: self.adjust.set_value(0.0000000000000) #MORE ZEROS SEEMS TO WORK LOL



	def on_scroll_up (self):
		if self.expand2 == 'Use a scrollbar' and int(self.value)-1 >= 0:
			self.adjust.set_value(int(self.value)-1)

	def on_scroll_down (self):
		if self.expand2 == 'Use a scrollbar' and int(self.value)+1 < self.fullscroll :
			self.adjust.set_value(int(self.value)+1)

	def update_scrollbar(self):
		self.fullscroll = round(((self.icon_num/self.sb_row)) / self.sb_column)
		try:
			if self.fullscroll/((float(self.icon_num)/float(self.sb_row)) / self.sb_column) < 1:
				self.fullscroll = self.fullscroll+1
		except: self.fullscroll = 1
		self.increment = 1
		if self.fullscroll <= 1: 
			self.hide_scrollbar()
			self.adjust.set_value(0.0000000000000)
		else:
			self.adjust.set_upper(self.fullscroll)
			self.show_scrollbar()
			self.scrollbarbox.move(self.scrollbar,int((self.width-self.border_size-self.shadow_size)*self.scale)-24 , int((self.banner_size+9)*self.scale))
			self.scrollbar.set_size_request(-1,int((self.height-self.banner_size*2)*self.scale))
		self.value = self.adjust.get_value()
		if len(str(self.value)) < 8:
			self.adjust.set_value(int(self.value))
			self.show_start = int(self.value * self.sb_row* self.sb_column)
			self.show_end = int(self.value * self.sb_row* self.sb_column + (self.sb_row* self.sb_column))
			if self.show_end > self.icon_num: self.show_end = self.icon_num
	

	
	def show_scrollbar(self):
		if str(self.scrollbar.flags()).find('GTK_MAPPED') == -1:

			self.scrollbarbox.put(self.scrollbar,0,0)
			
			self.scrollbar.set_size_request(-1,int((self.height-self.banner_size*2)*self.scale))
			self.scrollbarbox.show()
			self.scrollbar.show()
			self.width = int(((self.icon_size * 2 * self.rows) + ((self.border_size+self.shadow_size)*2)+15 ) + 24/self.scale)


	def hide_scrollbar(self):
		self.scrollbar.hide()
		self.width = int(((self.icon_size * 2 * self.rows) + ((self.border_size+self.shadow_size)*2)+15 ))
		#self.width = int(self.icon_size * 2 * self.rows + (self.border_size+self.shadow_size)*2+10)
		#self.width = int((self.icon_size * 2 * self.rows) +10 + (self.border_size+self.shadow_size)*2)
		self.init_buffers()
		self.redraw_background()
		self.redraw_foreground()
		self.update()
		self.scrollbar.hide()

#-------------------------------------------
# SAVE THEME METHODS
#-------------------------------------------


	def show_edit_dialog(self):
		# create dialog
		dialog = gtk.Dialog("Theme name to save?", self.window)
		dialog.resize(300, 100)
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, 
			gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		entrybox = gtk.Entry()
		entrybox.set_text(str('New theme name...'))
		dialog.vbox.add(entrybox)
		entrybox.show()	
		# run dialog
		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			try:
				os.mkdir(os.environ['HOME'] + '/.screenlets/MultiFolderView/themes/' + entrybox.get_text())
			except:
				screenlets.show_message(None,'Theme not created')
			f = open(os.environ['HOME'] + '/.screenlets/MultiFolderView/themes/' + entrybox.get_text() + '/theme.conf' , 'w')
			if f:
				f.write('[Theme]' + '\n')
				f.write('name	= ' + entrybox.get_text() +'\n')
				f.write('author	= ' + os.environ['USER'] +'\n')
				f.write('versio = ' + '1.0' + '\n')
				f.write('info	= ' + 'Created automaticly' + '\n')
				f.write('[Options]' + '\n')
				f.write('shadow_color=' + str(self.shadow_color).replace('[','(').replace(']',')') + '\n')
				f.write('pericons=' + str(self.pericons) + '\n')
				f.write('height=' + str(self.height) + '\n')
				f.write('font=' +  str(self.font) + '\n')
				f.write('title_font=' + str(self.title_font) + '\n')
				f.write('frame_color=' + str(self.frame_color).replace('[','(').replace(']',')') + '\n')
				f.write('width=' + str(self.width) + '\n')
				f.write('border_color=' + str(self.border_color).replace('[','(').replace(']',')') + '\n')
				f.write('show_title=' + str(self.show_title) + '\n')
				f.write('expand=' + str(self.expand) +'\n')
				f.write('banner_size=' + str(self.banner_size) + '\n')

				f.write('border_size_selected=' + str(self.border_size_selected) + '\n')
				f.write('frame_color_selected=' + str(self.frame_color_selected).replace('[','(').replace(']',')') + '\n')
				f.write('shadow_size=' + str(self.shadow_size) + '\n')
				f.write('border_size=' + str(self.border_size) + '\n')

				f.write('rounda=' + str(self.rounda) + '\n')
				f.write('color_text=' + str(self.color_text).replace('[','(').replace(']',')') + '\n')
				f.write('color_title=' + str(self.color_title).replace('[','(').replace(']',')') + '\n')
				f.write('icon_size=' + str(self.icon_size) + '\n')
				f.write('full_path=' + str(self.full_path) + '\n')

				f.close()
				screenlets.show_message(None,'Theme saved in ~/.screenlets/MultiFolderView/themes')
		dialog.hide()
		

#-------------------------------------------
# GET FILE LIST METHODS
#-------------------------------------------

	def populate_list(self, path):
		'''	Generate the list of GFile and GFileInfos
		'''		

		parent = gio.File(path)
		self.files_list_show = []
		self.icon_num = 0
		ldir = os.listdir(path)
		ldir.sort(key=str.upper)
		for elem in ldir:
			
			if os.path.isdir(path + '/' + elem):
				Gfile = gio.File(os.path.join(path,elem))
				tuble = [Gfile, Gfile.query_info("standard::*"), []]
				name = tuble[1].get_name()
				if not tuble[1].get_is_hidden() and not tuble[1].get_is_backup():
					if tuble not in self.files_list_show:
						if name not in self.exclude:
							if self.showbyex != '':
								if name.endswith(self.showbyex): 
									self.files_list_show.append(tuble)
									self.icon_num =  self.icon_num +1
							else:
								self.files_list_show.append(tuble)
								self.icon_num =  self.icon_num +1
				
				tuble = False
				Gfile = False
		for elem in ldir:
			
			if not os.path.isdir(path + '/' + elem):
				Gfile = gio.File(os.path.join(path,elem))
				tuble = [Gfile, Gfile.query_info("standard::*"), []]
				name = tuble[1].get_name()
				if not tuble[1].get_is_hidden() and not tuble[1].get_is_backup() and not name.lower().endswith('lnk'):
					if tuble not in self.files_list_show:
						if name not in self.exclude:
							if self.showbyex != '':
								if name.endswith(self.showbyex): 
									self.files_list_show.append(tuble)
									self.icon_num =  self.icon_num +1
							else:
								self.files_list_show.append(tuble)
								self.icon_num =  self.icon_num +1

				
				tuble = False
				Gfile = False		
		parent = False

		#list_files.sort(key=str.lower)
		if self.expand == 'horizontal':
			if self.expand2 == 'Use a scrollbar':
				self.rows = self.sb_row #self.pericons
				self.columns = self.sb_column #int(self.icon_num)/int(self.pericons)
			else:
				self.rows = self.pericons
				self.columns = int(self.icon_num)/int(self.pericons)
				self.columns = int(round(self.columns + 0.5))
				if int((self.columns-1) * self.pericons) == self.icon_num:
					self.columns = self.columns -1	
		else:
			if self.expand2 == 'Use a scrollbar':
				self.rows = self.sb_row #self.pericons
				self.columns = self.sb_column #int(self.icon_num)/int(self.pericons)
			else:

				self.columns = self.pericons
				self.rows = int(self.icon_num)/int(self.pericons)
				self.rows = int(round(self.rows + 0.5))
				if int((self.columns) * (self.rows-1)) == self.icon_num:
					self.rows = self.rows -1

		if self.expand2 == 'Use a scrollbar':
			self.width = int((self.icon_size * 2 * self.rows + ((self.border_size+self.shadow_size)*2)+15 ) + 24/self.scale)
		else:
			self.width = int(((self.icon_size * 2 * self.rows) + ((self.border_size+self.shadow_size)*2)+15 ))
		self.height = int(self.icon_size * 2 * self.columns + self.banner_size + (self.border_size+self.shadow_size)*2+15)

		x = 0
		gc.collect()
	
#-------------------------------------------
# DESKTOP FILES METHODS
#-------------------------------------------

	def gnomedesk_get(self,path):
		f = open(path, "r")
		tmp = f.readlines(2000)
		f.close()
		ico = ''
		exe = ''
		for line in tmp:
			if line.startswith('Icon='):
				ico = line.replace('Icon=','').replace('\n','')

			elif line.startswith('Exec='):
				exe = line.replace('Exec=','').replace('\n','')

			elif line.startswith('Name='):
				name = line.replace('Name=','').replace('\n','')
			

		return ico,exe,name

	def gnomedesk_get_name(self,path):
		return self.gnomedesk_get(path)[2]

	def gnomedesk_get_icon(self,path):
		return self.gnomedesk_get(path)[0]


	def gnomedesk_get_exe(self,path):
		return self.gnomedesk_get(path)[1]

#-------------------------------------------
# FILE INFO METHODS
#-------------------------------------------

	def generate_icon_names(self, tuble):
		''' 	Generate a list of icons names from a [GFile, GFileInfo, [x,y]]
		'''

		self.info = tuble[1]
		#print gio.desktop_app_info_new_from_filename (tuble[0].get_path())
		self.Gicon = self.info.get_icon()
		self.uri = tuble[0].get_path()
		self.icon_names = self.Gicon.get_names()
      		
		if self.showthumb:
      			
			self.info = tuble[0].query_info(gio.FILE_ATTRIBUTE_THUMBNAIL_PATH, gio.FILE_QUERY_INFO_NONE)
			thumbfile = self.info.get_attribute_as_string(gio.FILE_ATTRIBUTE_THUMBNAIL_PATH)
			if thumbfile:
				path = thumbfile.replace('file://','')
				if os.path.exists(path): return path
			#if GNOMEUI:
			#	icon, isthumbnail = self.thumbnailer.CheckWorkList(tuble[0].get_uri())
			#	if not icon:
			#		self.thumbnailer.AddToWorkList(tuble[0].get_uri(), tuble[1].get_content_type())
			#	else:
			#		if isthumbnail:
			#			return icon


		#icon = (self.thumbnailer.Process(tuble[0].get_uri(), tuble[1].get_content_type())).replace('file://','')
		#return icon

		if str(self.uri).lower().endswith('.desktop'):

			if GNOMEDESK:
				self.ico = gnomedesktop.item_new_from_file(self.uri, gnomedesktop.LOAD_ONLY_IF_EXISTS).get_string(gnomedesktop.KEY_ICON)

			else:
				self.ico = self.gnomedesk_get_icon(self.uri)
			if self.ico != None:
				if os.path.isfile(self.ico):
					return self.ico
				else:
					
					self.ico = self.ico.replace('.png','').replace('.svg','').replace('.xpm','')
				self.icon = self.icontheme.lookup_icon(self.ico,self.icon_size,self.icon_size)
				if self.icon:
					return self.icon.get_filename()	
		else:
			self.iconinfo = self.icontheme.choose_icon(self.icon_names, self.icon_size,gtk.ICON_LOOKUP_USE_BUILTIN);
			if self.iconinfo:
				return self.iconinfo.get_filename()		
			for x in range(0,len(self.icon_names)-1):
				xx = len(self.icon_names)-1-x
				if self.icontheme.has_icon(self.icon_names[xx]):
					return self.icon_names[xx]
		return 'gtk-execute'
	

	def icons_changed(self, event):
		self.redraw_foreground()
		self.update()


	def generate_file_name(self, tuble):
		'''	Generate the name of a file from a (GFile, GFileInfo)
		'''
		info = tuble[1].get_name()
		if info.lower().endswith('.desktop'):
			info = self.gnomedesk_get_name(tuble[0].get_path())#info[:info.lower().find('.desktop')]
		return info


	def set_path_to_parent(self):
		if self.folder_path_current is not bool:
			self.folder_path_current = os.path.split(self.folder_path_current)[0]
			self.update_path()

	def monitor_path(self, path):
		''' 	Monitor the path for modified files/directory
		''' 	
		if self.current_file:
			self.current_file.disconnect_by_func(self.monitor_callback)

		self.current_file = gio.File(self.folder_path_current).monitor_directory()
		self.current_file.connect("changed", self.monitor_callback)
		

	def monitor_callback(self, monitor, Gfile, event, data=None):
		try:
			tuble = [Gfile, Gfile.query_info("standard::*"), []]
		except:
			self.update_path()
			#self.populate_list(self.folder_path_current)
			if self.expand2 == 'Use a scrollbar':
				self.update_scrollbar()	
			self.init_buffers()
			self.redraw_background()
			self.redraw_foreground()
			self.update()
			if self.expand2 == 'Use a scrollbar':
				self.update_scrollbar()	

			return True
		if not tuble[1].get_is_hidden() and not tuble[1].get_is_backup():
			self.update_path()
			#self.populate_list(self.folder_path_current)
			if self.expand2 == 'Use a scrollbar':
				self.update_scrollbar()	
			self.init_buffers()
			self.redraw_background()
			self.redraw_foreground()
			self.update()
			if self.expand2 == 'Use a scrollbar':
				self.update_scrollbar()	

		#self.show_notification("Changes !")
		return True


	def update_path_from_settings(self):
		self.folder_path_current = self.folder_path
		self.monitor = self.monitor_path(self.folder_path_current)
		self.update_path()
		self.init_buffers()
		self.redraw_background()
		self.redraw_foreground()
		self.update()


	def update_path(self):
		''' Update path from settings
		'''
		
		if self.folder_path_current is not bool:
			try:
				self.populate_list(self.folder_path_current)
			except:
				self.populate_list(os.path.join(os.path.expanduser("~")))
			self.monitor = self.monitor_path(self.folder_path_current)		
			
	
		self.update()
		gc.collect()


#-------------------------------------------
# MOUSE CLICKS METHODS
#-------------------------------------------

	def click_callback(self):
		'''	Callback when click
		'''
		elem = self.get_selected_element()
		if elem:
			if str(elem[0].get_path()).lower().endswith('.desktop'):
				uri = elem[0].get_path()
				if GNOMEDESK:
					item = gnomedesktop.item_new_from_file(uri, gnomedesktop.LOAD_ONLY_IF_EXISTS)
					run = item.get_string(gnomedesktop.KEY_EXEC)
				else:
					run = self.gnomedesk_get_exe(uri)
				os.system(run + ' &')
			else:
				xdg_open(chr(34) + elem[0].get_path() + chr(34))
			return



	def right_clicl_callback(self):
		self.menu = gtk.Menu()
		elem = self.get_selected_element()
		if elem:
			apps = gio.app_info_get_all_for_type(elem[1].get_content_type())
			self.apps_list = []
			self.apps_execs =  {}
			for app in apps:
				self.apps_list.append(app.get_name())
				self.apps_execs[app.get_name()] = app.get_executable() + ' ' + '"' +elem[0].get_path() + '"'
			
			#self.add_submenuitem("Open with","Open with",self.apps_list)
			#self.add_submenuitem("Actions", "Actions",['Delete'])
			#self.add_menuitem("Delete", "Delete")
			self.add_menuitem("Delete", "Apagar")
			#self.add_menuitem("Delete", "Supprimer")
			#self.add_menuitem("Delete", "Borrar")
			self.cp = "file://" +  elem[0].get_path()
		#self.add_menuitem("Rename", "Rename Folder")
		self.add_menuitem("Rename", "Alterar Nome da Pasta")
		#self.add_menuitem("Rename", "Renommer le Dossier")
		#self.add_menuitem("Rename", "Renombrar Carpeta")
		#self.add_menuitem("export", "Save Theme")
		#self.add_default_menuitems()"""



	def on_mouse_down(self, event):
		"""Called when a buttonpress-event occured in Screenlet's window. 
		Returning True causes the event to be not further propagated."""
		self.set_current_cursor(event,self.window) 
		if event.type == gtk.gdk.BUTTON_PRESS:
			if event.button == 1:
				if self.lock_position:
					self.clicked = True
				else:
					self.click_callback()
	
		
					#self.update_scrollbar()	
					self.clicked = False
					return False
	
			elif event.button == 3:
				self.clicked = True
				self.right_clicl_callback()
		elif event.type == gtk.gdk._2BUTTON_PRESS:
			elem = self.get_selected_element()
			#if not elem and self.mousey < self.banner_size:
				#os.system('xdg-open ' +chr(34) +self.folder_path_current + chr(34))


	def on_mouse_up(self, event):

		self.set_current_cursor(event,self.window) 
		if event.type == gtk.gdk.BUTTON_RELEASE:
			if event.button == 1:
				if self.lock_position:
					self.click_callback()
	
					#self.update_scrollbar()	
					self.clicked = False
					return False
				else:
					self.clicked = True

	
			elif event.button == 3:
				self.clicked = True
				self.right_clicl_callback()

		elif event.type == gtk.gdk._2BUTTON_PRESS:
			
			elem = self.get_selected_element()
			#if not elem:
				#os.system('xdg-open ' +chr(34) +self.folder_path_current + chr(34))
				


#-------------------------------------------
# OTHER SCREENLETS METHODS
#-------------------------------------------

	def adjust_screenlets_position(self, w, h):
		#self.scale = scale
		#padding = int(100 * self.scale)
		#offset_x = w - (padding + int(self.width * self.scale * 2))
		#offset_y = h - (padding + int(self.height * self.scale * 2))		
		cpt_w = w / 2
		cpt_h = h / 2
		if self.id == 'MultiFolderView1':
			screenlets.Screenlet.__setattr__(self, 'x', cpt_w - self.width - 10)
			screenlets.Screenlet.__setattr__(self, 'y', cpt_h - self.height - 10)
		elif self.id == 'MultiFolderView2':
			screenlets.Screenlet.__setattr__(self, 'x', cpt_w + 10)
			screenlets.Screenlet.__setattr__(self, 'y', cpt_h - self.height - 10)
		elif self.id == 'MultiFolderView3':
			screenlets.Screenlet.__setattr__(self, 'x', cpt_w - self.width - 10)
			screenlets.Screenlet.__setattr__(self, 'y', cpt_h + 10)
		elif self.id == 'MultiFolderView4':
			screenlets.Screenlet.__setattr__(self, 'x', cpt_w + 10)
			screenlets.Screenlet.__setattr__(self, 'y', cpt_h + 10)	

	def adjust_screenlets_size(self, w, h):
		if (w <= 800 and h <= 600):
			screenlets.Screenlet.__setattr__(self, 'icon_size', 44)
		elif (w <= 1024 and h <= 768):
        		screenlets.Screenlet.__setattr__(self, 'icon_size', 60)
        	elif (w <= 1280 and h <= 900):
            		screenlets.Screenlet.__setattr__(self, 'icon_size', 60)
 	        else:
			screenlets.Screenlet.__setattr__(self, 'icon_size', 80)
		self.adjust_screenlets_position(w, h)
		
	def adjust_screenlets(self):
		pr = subprocess.Popen("xrandr -d :0 | grep \'+0+0\'", shell=True, stdout=subprocess.PIPE)
		arr = pr.stdout.read().split(' ')[2].split('x')
		w = int(arr[0])
		h = int(arr[1][:-4])
		if self.screen_width != w or self.screen_height != h:
			self.screen_width = w
			self.screen_height = h
			self.adjust_screenlets_size(w, h)
			#self.update()
			

	def setFolders(self):
		folders = getFolders(CENTRAL_FOLDER)
		if (self.numId <= len(folders)):
			self.folderName = folders[self.numId-1]
			screenlets.Screenlet.__setattr__(self, 'folder_path', os.path.join(CENTRAL_FOLDER, self.folderName))		
		
	def on_init(self):
		elem = self.get_selected_element()
		if elem:
			#screenlets.Screenlet.__setattr__(self, 'periconon_inits', 3)
			apps = gio.app_info_get_all_for_type(elem[1].get_content_type())
			self.apps_list = []
			self.apps_execs =  {}
			for app in apps:
				self.apps_list.append(app.get_name())	
				self.apps_execs[app.get_name()] = app.get_executable() + ' ' + '"' +elem[0].get_path() + '"'
				#self.add_submenuitem("Open with","Open with",self.apps_list)
				
			self.add_submenuitem("Actions", "Actions", ['Copy','Paste','Delete'])
			self.cp = "file://" +  elem[0].get_path()
		screenlets.Screenlet.__setattr__(self, 'keep_below', True)
		screenlets.Screenlet.__setattr__(self, 'keep_above', False)
		self.setFolders()
		gobject.timeout_add(1000, self.finish)
		self.update_path_from_settings()

	def finish(self):
		self.window.set_keep_below(1)
		self.add_menuitem("export", "Save Theme")
		#self.add_default_menuitems()
		self.update_path_from_settings()
		if self.expand2 == 'Use a scrollbar':
			self.update_scrollbar()
			self.scrollbar.show()
			self.value = self.adjust.get_value()
			self.redraw_background()
			self.redraw_foreground()
			self.update()
			self.scrollbar.modify_bg(gtk.STATE_NORMAL,  gtk.gdk.Color(self.frame_color[0]*65535,self.frame_color[1]*65535,self.frame_color[2]*65535,0)) # cor da barra maior
			self.scrollbar.modify_bg(gtk.STATE_SELECTED,  gtk.gdk.Color(self.color_title[0]*65535,self.color_title[1]*65535,self.color_title[2]*65535,0)) # cor da barra menor
			self.scrollbar.modify_bg(gtk.STATE_PRELIGHT,  gtk.gdk.Color(self.color_title[0]*65535,self.color_title[1]*65535,self.color_title[2]*65535,0)) # cor da seta a funcionar
			self.scrollbar.modify_bg(gtk.STATE_ACTIVE,  gtk.gdk.Color(self.color_title[0]*65535,self.color_title[1]*65535,self.color_title[2]*65535,0)) # cor da seta a funcionar
			self.scrollbar.modify_bg(gtk.STATE_INSENSITIVE,  gtk.gdk.Color(self.frame_color[0]*65535,self.frame_color[1]*65535,self.frame_color[2]*65535,0)) # cor da seta a nao funcionar
			self.scrollbar.modify_fg(gtk.STATE_NORMAL,  gtk.gdk.Color(self.color_title[0]*65535,self.color_title[1]*65535,self.color_title[2]*65535,0)) 

		return False
		


	def draw_text(self, ctx, text, x, y,  font, size, width, allignment=pango.ALIGN_LEFT,alignment=None,justify = False,weight = 0, ellipsize = pango.ELLIPSIZE_NONE,title=False):
		"""Draws text"""
		ctx.save()
		ctx.translate(x+1, y+1)
		if self.p_layout == None :
	
			self.p_layout = ctx.create_layout()
		else:
			
			ctx.update_layout(self.p_layout)
		if self.p_fdesc == None:self.p_fdesc = pango.FontDescription()
		else: pass
		self.p_fdesc.set_family(pango.FontDescription(font).get_family())
		self.p_fdesc.set_size(size * pango.SCALE)
		self.p_fdesc.set_weight(weight)
		self.p_fdesc.set_style(pango.FontDescription(font).get_style())
		self.p_layout.set_font_description(pango.FontDescription(font))
		#self.p_layout.set_font_description(self.font)
		self.p_layout.set_width(width * pango.SCALE)
		self.p_layout.set_alignment(allignment)
		if alignment != None:self.p_layout.set_alignment(alignment)
		self.p_layout.set_justify(justify)
		self.p_layout.set_wrap(pango.WRAP_WORD_CHAR)
		#self.p_layout.set_ellipsize(ellipsize)
		#self.p_layout.set_height(20)#doesnt work in pygtk yet, check latter
		self.p_layout.set_ellipsize(pango.ELLIPSIZE_NONE)

		
		if title == True:
	
			if self.banner_size != self.p_layout.get_size()[1]/pango.SCALE + 10 :
				self.banner_size = self.p_layout.get_size()[1]/pango.SCALE + 10 

		
		#print self.p_layout.get_text(),self.p_layout.get_size()[0]/pango.SCALE
		self.p_layout.set_markup(text)
		#Workarround for non set_height
			
		if self.p_layout.get_line(2):
			while True: 
				if self.p_layout.get_line(2):
					text = text[:-1]
					self.p_layout.set_markup(text)
				else:
					text = text[:-3] + '...'
					self.p_layout.set_markup(text)
					break
		#print self.p_layout.get_pixel_size()[1]*self.scale , ((self.icon_size * 1.8)-(self.icon_size * 1.2+1))*self.scale
		if self.p_layout.get_pixel_size()[1]*self.scale > ((self.icon_size * 1.8)-(self.icon_size * 1.2+1))*self.scale:
			self.p_layout.set_ellipsize(ellipsize)

		ctx.set_source_rgba(1-self.color_text[0],1-self.color_text[1],1-self.color_text[2],0.5)
		ctx.show_layout(self.p_layout)
		try:
			ctx.set_source_rgba(*self.color_text)
		
			ctx.translate(-1, -1)
			ctx.show_layout(self.p_layout)
			ctx.restore()
		except:pass



	def __setattr__(self, name, value):
		# call Screenlet.__setattr__ in baseclass (ESSENTIAL!!!!)
		if (name == 'folder_path' and value == False): 
			pass
		elif (name == 'banner_size' and value == self.banner_size): 
			pass
		else:
			screenlets.Screenlet.__setattr__(self, name, value)

		if self.has_started:

			if name in  ['folder_path','expand','pericons','showbyex','banner_size','sb_row','sb_column','show_title','showbyex']:
				self.update_path_from_settings()
				if self.expand2 == 'Use a scrollbar':
					self.update_scrollbar()	

			if name in ['border_color','rounda','image_filename','show_back', 'shadow_color','frame_color','title_font','color_title']:
				self.redraw_background()
				self.update()

			if name in ['icon_size','color_text','font']:
				self.update_path_from_settings()
				self.init_buffers()
				self.redraw_foreground()
				self.update()

			if name in ['roundb','frame_color_selected','border_size_selected','shadow_size','border_size']:
				self.redraw_background()
				self.redraw_foreground()
				self.update()

			if name == 'expand2':
				if not value == 'Use a scrollbar':
					self.hide_scrollbar()
				
				self.update_path_from_settings()
				self.init_buffers()
				self.redraw_background()
				self.redraw_foreground()
				self.update()
				if value == 'Use a scrollbar':
					self.update_scrollbar()	


		if name == 'show_title' and value == False: self.banner_size = 0



	def get_style_color(self):
		''' 	Get default color for background
			#TODO Use this
		'''
		style = gtk.Style().bg[gtk.STATE_NORMAL]
		return style

	def get_user_string(self, message, title=''):
		dialog = gtk.Dialog(title, self.window)
		dialog.resize(300, 110)
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		label = gtk.Label(message)
		entrybox = gtk.Entry(100)
		entrybox.set_width_chars(15)
		entrybox.set_text(str(self.folderName))
		dialog.vbox.pack_start(label, False, False, 10)
		label.show()
		dialog.vbox.pack_end(entrybox, False, False, 5)
		entrybox.show()
				
		dialog.set_default_response(gtk.RESPONSE_OK)

		response = dialog.run()
		text = entrybox.get_text()
		dialog.destroy()
		if (response == gtk.RESPONSE_OK and text != ''):
			return text
		else:
			return None
	
	#def get_string_user(self, message, title=''):
	#	dialogWindow = Gtk.MessageDialog(self, 
	#					Gtk.DialogFlags.MODAL | gtk.DialogFlags.DESTROY_WITH_PARENT,
	#					Gtk.MessageType.QUESTION,
	#					Gtk.ButtonsType.OK_CANCEL,
	#					message)
	#	dialogWindow.set_title(title)
	#	
	#	dialogBox = dialogWindow.get_content_area()
	#	userEntry = Gtk.Entry(100)
	#	dialogBox.pack_end(userEntry, False, False, 0)
#
#		dialogWindow.show_all()
#		response = dialogWindow.run()
#		text = userEntry.get_text()
#		dialogWindow.destroy()
#		
#		if (response == Gtk.ResponseType.OK and text != ''):
#			return text
#		else:
#			return None

	def on_menuitem_select(self, id):
		"""handle MenuItem-events in right-click menu"""
		if id == 'Rename':
			#folder = self.get_user_string('Choose the folder\'s new name', 'Rename folder')
			folder = self.get_user_string('Escolha o novo nome para a apasta', 'Alterar nome da pasta')
			#folder = self.get_user_string('Elegir un nuevo nombre para la carpeta', 'Renombrar carpeta')
			#folder = self.get_user_string('Choisir le nouveau nom du dossier', 'Renommer le dossier')
			folderList = getFolders(CENTRAL_FOLDER)
			if (folder in folderList):
				md = gtk.MessageDialog(self.window, 
						       gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, 
						       #gtk.BUTTONS_CLOSE, "A folder named " + folder + " already exists.")
						       gtk.BUTTONS_CLOSE, "Já existe uma pasta chamada " + folder + " .")
						       #gtk.BUTTONS_CLOSE, "Ya existe una carpeta con el nombre " + folder + " .")
						       #gtk.BUTTONS_CLOSE, "Um dossier nommé " + folder + " existe dejá.")
        			md.run()
				md.destroy()
				# Error, folder already exists
			else:
				os.system('mv ' + '\"' + self.folderName + '\"' + ' ' + '\"' +  folder + '\"')
				self.folderName = folder
				self.folder_path_current = os.path.join(CENTRAL_FOLDER, self.folderName)			
				self.update_path()			
			# Do a mv
		elif id == 'Delete':
			#if screenlets.show_question(self,"Delete " + self.cp + ' ?'):
			if screenlets.show_question(self,"Apagar " + self.cp + ' ?'):
			#if screenlets.show_question(self,"Borrar " + self.cp + ' ?'):
			#if screenlets.show_question(self,"Supprimer " + self.cp + ' ?'):
				os.system('rm -rf ' + chr(34) + self.cp.replace('file://','') + chr(34))
				self.update_scrollbar()

		if id in self.apps_list:
			os.system(self.apps_execs[id]+ ' &')
		self.clicked = False
		self.on_mouse_leave(0)


	def update(self):
		self.adjust_screenlets()
		self.redraw_canvas()
		
		return True # keep on running this event

		
	def on_mouse_enter (self, event):
		self.clicked = False
		self.redraw_canvas()
		#self.show_tip()


	def on_mouse_leave (self, event):
		"""Called when the mouse leaves the Screenlet's window."""

		if not self.clicked:

			self.cursor_position = [-1,-1]
			self.redraw_canvas()
			self.hide_tip()#self.timer1 = gobject.timeout_add(2000, self.hide_tip)

	def motion_notify_event(self, widget, event):
		"""Called when the mouse moves in the Screenlet's window."""
		#if event.window == self.ww:
		before_cursor = self.cursor_position
		self.set_current_cursor(event,widget)
		if before_cursor <> self.cursor_position:
			self.hide_tip()
			self.show_tip()
			self.update()


	def set_current_cursor(self, event,widget):
		self.mousex,self.mousey = widget.get_pointer()
		self.cursor_position= [int(self.mousex / self.scale), int(self.mousey / self.scale)]
		if self.cursor_position[1] < self.banner_size:
			self.cursor_position = [-1,-1]
		else:
			self.cursor_position  = [int(self.cursor_position[0]-self.border_size - self.shadow_size-5) / int((self.icon_size * 2)), 
				int(self.cursor_position[1]-self.banner_size-self.border_size - self.shadow_size-5) / int( int((self.icon_size*2 )))]
			

	def get_x(self, situation):
		return situation[2][0] * self.icon_size * 2

	def get_y(self,situation):
		return situation[2][1] * self.icon_size * 2 + self.banner_size

	def init_buffers (self):
		"""(Re-)Create back-/foreground buffers"""
		if self.width >0:
			if self.expand2 == 'Use a scrollbar':self.scrollbar.hide()
			self.__buffer_back = gtk.gdk.Pixmap(self.window.window, 
				int(self.width * self.scale), int(self.height * self.scale), -1)
			self.__buffer_fore = gtk.gdk.Pixmap(self.window.window, 
				int(self.width * self.scale), int(self.height * self.scale), -1)
			if self.expand2 == 'Use a scrollbar':self.scrollbar.show()
		else: print 'SHIT'



	def redraw_background(self):
		# create context
		self.ctx_back = self.__buffer_back.cairo_create()
		# clear context
		self.clear_cairo_context(self.ctx_back)
		# compose background
		self.ctx_back.set_operator(cairo.OPERATOR_OVER)
		self.ctx_back.scale(self.scale, self.scale)
		self.ctx_back.set_source_rgba(*self.frame_color)

		if self.full_path == True:
			bt = str(self.folder_path_current)
		else:
			bt = str(self.folder_path_current).split('/')[len(str(self.folder_path_current).split('/'))-1]

		if self.show_back and self.image_filename != "":
		
			self.draw_scaled_image(self.ctx_back,0 ,0, urllib.unquote(self.image_filename.replace('file://','')), self.width-10+self.shadow_size, self.height-10+self.shadow_size)	
		else:
			self.ctx_back.set_source_rgba(*self.frame_color)
			self.draw_rectangle_advanced (self.ctx_back, 0, 0, self.width-(self.border_size + self.shadow_size)*2, self.height-(self.border_size + self.shadow_size)*2,
			rounded_angles=(self.rounda,self.rounda,self.rounda,self.rounda), fill=True, border_size=self.border_size, border_color=(self.border_color[0],self.border_color[1],self.border_color[2],self.border_color[3]),
			shadow_size=self.shadow_size, shadow_color=(self.shadow_color[0],self.shadow_color[1],self.shadow_color[2],self.shadow_color[3]))

			self.ctx_back.set_source_rgba(1-self.color_title[0],1-self.color_title[1],1-self.color_title[2],0.3)
			self.draw_line(self.ctx_back,10,self.banner_size +2 ,self.width-30,0,line_width = 1,close=False,preserve=False)
			self.ctx_back.set_source_rgba(*self.color_title)
			self.draw_text(self.ctx_back, bt, 			# self.ctx_back, texte
				10, 			# x
				10,			# y
				self.title_font, 				# font
				int(self.banner_size * 0.4) , int(self.width-10) ,		# size, width
				alignment=pango.ALIGN_LEFT, ellipsize=pango.ELLIPSIZE_END,title=True)
			self.draw_line(self.ctx_back,10,self.banner_size +2 ,self.width-30,0,line_width = 1,close=False,preserve=False)


	def redraw_foreground(self):

		# create context from fg-buffer
		
		self.ctx_fore = self.__buffer_fore.cairo_create()
		self.clear_cairo_context(self.ctx_fore)
		self.ctx_fore.scale(self.scale, self.scale)
			#Draw the icon
		self.ctx_fore.set_source_rgba(*self.frame_color)
		self.ctx_fore.translate(self.border_size + self.shadow_size+7,self.border_size + self.shadow_size+5)
		self.x_position = self.y_position = 0
		self.list = self.files_list_show
		if self.expand2 == 'Use a scrollbar':
			self.list =  self.files_list_show[self.show_start:self.show_end]

		for elem in self.list:
			if self.expand == 'horizontal':
				#Update the position of the file
				elem[2] = [self.x_position, self.y_position]
	
				if self.x_position == self.rows-1:# If we are at the end of the row, 
								# jump to the next row
	
					self.x_position = int(0)
					self.y_position = int(self.y_position) + 1
	
				else: 
					self.x_position = int(self.x_position) + 1
			else:
				#Update the position of the file

				elem[2] = [self.x_position, self.y_position]
				if self.y_position == self.columns -1:# If we are at the end of the row, 
								# jump to the next row
	
					self.x_position = int(self.x_position) + 1
					self.y_position = int(0)
	
				else: 
					self.y_position = int(self.y_position) + 1
			ico = self.generate_icon_names(elem)
			if os.path.isfile(ico):
			#if ico.lower().endswith('png') or ico.lower().endswith('svg') or ico.lower().endswith('xpm'):#xpm",
				w,h = self.get_image_size(ico)
				if int(w) != int(h):
					if int(w) != self.icon_size and int(h) != self.icon_size:
						if  (w*self.icon_size)/h > (self.icon_size * 1.8*self.scale +3):
							wi = self.icon_size * 1.8*self.scale +3-5
						else:
							wi = (w*self.icon_size)/h
						self.draw_scaled_image(self.ctx_fore, 						# self.ctx_fore
							self.get_x(elem) + (self.icon_size + (self.icon_size-wi))* 0.5 ,	# x
							self.get_y(elem) + self.icon_size  * 0.2,ico,int(wi),self.icon_size)	
				else:
					if int(w) != self.icon_size and int(h) != self.icon_size:
						self.draw_scaled_image(self.ctx_fore, 						# self.ctx_fore
							self.get_x(elem) + self.icon_size * 0.5 ,	# x
							self.get_y(elem) + self.icon_size * 0.2,ico,self.icon_size,self.icon_size)	
	
					else:
						self.draw_image(self.ctx_fore, 						# self.ctx_fore
							self.get_x(elem) + self.icon_size * 0.5 ,	# x
							self.get_y(elem) + self.icon_size * 0.2,ico)				
			else:
				try:
					self.draw_icon(self.ctx_fore, 						# self.ctx_fore
						self.get_x(elem) + self.icon_size * 0.5 ,	# x
						self.get_y(elem) + self.icon_size * 0.2,		# y
						ico, 			# icon name
						self.icon_size, self.icon_size)	
				except:
					self.draw_icon(self.ctx_fore, 						# self.ctx_fore
						self.get_x(elem) + self.icon_size * 0.5 ,	# x
						self.get_y(elem) + self.icon_size * 0.2,		# y
						gtk.STOCK_MISSING_IMAGE, 			# icon name
						self.icon_size, self.icon_size)	

			#Draw the text
			self.ctx_fore.set_source_rgba(*self.color_text)
			self.draw_text(self.ctx_fore, self.generate_file_name(elem), 		# self.ctx_fore, texte
				self.get_x(elem)+3, 			# x
				self.get_y(elem) + self.icon_size * 1.2+5,			# y
				self.font, 				# font
				int(self.icon_size * 0.18), int(self.icon_size * 2)-6 ,		# size, width
				alignment=pango.ALIGN_CENTER, ellipsize=pango.ELLIPSIZE_END)


	def on_draw(self, ctx):
		
		ctx.save()
		self.adjust_screenlets()
		if self.__buffer_back:
			ctx.set_operator(cairo.OPERATOR_OVER)
			ctx.set_source_pixmap(self.__buffer_back, 0, 0)
			ctx.paint()

		ctx.set_source_rgba(*self.frame_color_selected)
		elem = self.get_selected_element()
		if elem:
			self.draw_rectangle_advanced(ctx,			# ctx
				(self.get_x(elem)+self.border_size + self.shadow_size+5) ,		# x
				(self.get_y(elem)+self.border_size + self.shadow_size+5) ,		# y
				self.icon_size * 2 *self.scale , self.icon_size * 1.9  ,		# width, height
				rounded_angles=(self.roundb,self.roundb,self.roundb,self.roundb), fill=True,
				border_size=self.border_size_selected, border_color=(self.color_text[0],self.color_text[1],self.color_text[2],0.5))

			
		if self.__buffer_fore:
			ctx.set_operator(cairo.OPERATOR_OVER)
			ctx.set_source_pixmap(self.__buffer_fore, 0, 0)
			ctx.paint()

		ctx.restore()
		gc.collect()

	def on_draw_shape(self, ctx):
		#ctx.scale(self.scale, self.scale)
		#on_draw(ctx)
		ctx.set_source_rgba(0, 0, 0, 1)
 		ctx.rectangle(0, 0, self.width, self.height)
		ctx.fill()
		

# If the program is run directly or passed as an argument to the python
# interpreter then create a Screenlet instance and show it
if __name__ == "__main__":
	import screenlets.session
	screenlets.session.create_session(MultiFolderViewScreenlet)
